import sys
import time
import argparse

from powerful_pipes import read_json_from_stdin, write_json_to_stdout, \
    NotConnectedToPipe, report, REPORT_LEVEL


def main():
    parser = argparse.ArgumentParser(
        description='Ticker input data'
    )
    parser.add_argument('-b', '--banner',
                        default=False,
                        action="store_true",
                        help="displays tool banner")
    parser.add_argument('-t', '--tag',
                        required=True,
                        help="tag name")
    parsed = parser.parse_args()


    if parsed.banner:
        print(f"[*] Starting Powerful-pipes Ticker", flush=True, file=sys.stderr)


    try:
        for error, input_json in read_json_from_stdin():

            if error:
                report(
                    {},
                    log_level=REPORT_LEVEL.ERROR,
                    message=f"[Ticker] Error processing input JSON: {input_json}"
                )

            else:

                input_json.setdefault("_meta", {}).setdefault("chain", []).append(
                    {parsed.tag: {"executionTime": time.time()}}
                )

                write_json_to_stdout(input_json, force_flush=True)

    except NotConnectedToPipe:
        write_json_to_stdout({
            "_meta": {"chain": [{parsed.tag: {"executionTime": time.time()}}]}
        }, force_flush=True)

    except KeyboardInterrupt:
        ...


if __name__ == '__main__':
    main()



import random
import datetime

from powerful_pipes import read_from_stdin_by_file_ref, eprint, write_to_stdout_by_file_ref, report_exception


def main():

    for error, input_json in read_from_stdin_by_file_ref(auto_delete=False):

        if error:
            # Here you can manager the error. Most common error is that the
            # input is not a valid json.
            # eprint(...) function dumps the error message to the stderr
            eprint(message=f"Error processing input from stdin: {error}")
            continue

        try:
            input_json[str(random.randint(10, 1000))] = datetime.datetime.now()

        except Exception as e:
            report_exception(input_json, e)

        finally:
            # THIS STEP IS CRITICAL. If you don't put again in the stdout the
            # input data, following tools in the pipe chain wouldn't receive
            # the data
            write_to_stdout_by_file_ref(input_json)


if __name__ == '__main__':
    main()



import json

try:
    import orjson
except ImportError:
    ...

from .typing import JSON

def dump_json(obj: dict or list) -> str:
    """Dumps json using most efficient JSON library available"""
    try:
        json_lib = globals()["orjson"]
    except KeyError:
        json_lib = json

    try:
        dump_text = json_lib.dumps(obj)
    except TypeError as e:
        if "Dict" in str(e):
            # See: https://github.com/ijl/orjson#opt_non_str_keys
            dump_text = json_lib.dumps(obj, option=orjson.OPT_NON_STR_KEYS)
        else:
            dump_text = json.dumps(obj)

    if hasattr(dump_text, "decode"):
        dump_text = dump_text.decode()

    return dump_text

def read_json(data: str or bytes) -> JSON:
    """Reads json using most efficient JSON library available"""
    try:
        json_lib = globals()["orjson"]
    except KeyError:
        json_lib = json

    return json_lib.loads(data)


__all__ = ("dump_json", "read_json")
